@@ -6,14 +6,19 @@ use std::str::FromStr;
 
 use config::{Config, File};
 
+use anyhow::{anyhow, Result};
+use once_cell::sync::OnceCell;
 use serde::{Deserialize, Serialize};
 use serde_with::rust::string_empty_as_none;
 
 use super::UsageModel;
+use crate::commands::whoami::fetch_accounts;
 use crate::commands::{validate_worker_name, whoami, DEFAULT_CONFIG_PATH};
 use crate::deploy::{self, DeployTarget, DeploymentSet};
+use crate::settings::global_user::GlobalUser;
 use crate::settings::toml::builder::Builder;
 use crate::settings::toml::dev::Dev;
+use crate::settings::toml::durable_objects::DurableObjects;
 use crate::settings::toml::environment::Environment;
 use crate::settings::toml::kv_namespace::{ConfigKvNamespace, KvNamespace};
 use crate::settings::toml::route::RouteConfig;
@@ -34,7 +39,7 @@ pub struct Manifest {
     #[serde(rename = "type")]
     pub target_type: TargetType,
     #[serde(default)]
-    pub account_id: String,
+    pub account_id: LazyAccountId,
     pub workers_dev: Option<bool>,
     #[serde(default, with = "string_empty_as_none")]
     pub route: Option<String>,
@@ -55,27 +60,31 @@ pub struct Manifest {
     pub text_blobs: Option<HashMap<String, PathBuf>>,
     pub wasm_modules: Option<HashMap<String, PathBuf>>,
     pub triggers: Option<Triggers>,
+    pub durable_objects: Option<DurableObjects>,
     #[serde(default, with = "string_empty_as_none")]
     pub usage_model: Option<UsageModel>,
+    pub compatibility_date: Option<String>,
+    #[serde(default)]
+    pub compatibility_flags: Vec<String>,
 }
 
 impl Manifest {
-    pub fn new(config_path: &Path) -> Result<Self, failure::Error> {
+    pub fn new(config_path: &Path) -> Result<Self> {
         let file_name = config_path.file_name().unwrap().to_str().unwrap();
         let mut message = format!("{} not found", file_name);
         if config_path.to_str().unwrap() == DEFAULT_CONFIG_PATH {
             message.push_str("; run `wrangler init` to create one.");
         }
-        failure::ensure!(config_path.exists(), message);
+        anyhow::ensure!(config_path.exists(), message);
         let config = read_config(config_path)?;
 
         let manifest: Manifest = match config.try_into() {
             Ok(m) => m,
             Err(e) => {
                 if e.to_string().contains("unknown field `kv-namespaces`") {
-                    failure::bail!("kv-namespaces should not live under the [site] table in your configuration file; please move it above [site].")
+                    anyhow::bail!("kv-namespaces should not live under the [site] table in your configuration file; please move it above [site].")
                 } else {
-                    failure::bail!(e)
+                    anyhow::bail!(e)
                 }
             }
         };
@@ -88,9 +97,9 @@ impl Manifest {
     pub fn generate(
         name: String,
         target_type: Option<TargetType>,
-        config_path: &PathBuf,
+        config_path: &Path,
         site: Option<Site>,
-    ) -> Result<Manifest, failure::Error> {
+    ) -> Result<Manifest> {
         let config_file = &config_path.join("wrangler.toml");
         let config_template_str = fs::read_to_string(config_file).unwrap_or_else(|err| {
             log::info!("Error reading config template: {}", err);
@@ -125,22 +134,16 @@ impl Manifest {
          * as only toml-rs supports serde.
          */
 
-        let mut config_template_doc =
-            config_template_str
-                .parse::<toml_edit::Document>()
-                .map_err(|err| {
-                    failure::err_msg(format!(
-                        "toml_edit failed to parse config template. {}",
-                        err
-                    ))
-                })?;
+        let mut config_template_doc = config_template_str
+            .parse::<toml_edit::Document>()
+            .map_err(|err| anyhow!("toml_edit failed to parse config template. {}", err))?;
 
         config_template_doc["name"] = toml_edit::value(name);
         if let Some(default_workers_dev) = default_workers_dev {
             config_template_doc["workers_dev"] = toml_edit::value(default_workers_dev);
         }
         if let Some(target_type) = &target_type {
-            config_template_doc["target_type"] = toml_edit::value(target_type.to_string());
+            config_template_doc["type"] = toml_edit::value(target_type.to_string());
         }
         if let Some(site) = site {
             if config_template.site.is_none() {
@@ -193,7 +196,7 @@ impl Manifest {
 
     fn route_config(&self) -> RouteConfig {
         RouteConfig {
-            account_id: Some(self.account_id.clone()),
+            account_id: self.account_id.clone(),
             workers_dev: self.workers_dev,
             route: self.route.clone(),
             routes: self.routes.clone(),
@@ -201,7 +204,7 @@ impl Manifest {
         }
     }
 
-    pub fn get_deployments(&self, env: Option<&str>) -> Result<DeploymentSet, failure::Error> {
+    pub fn get_deployments(&self, env: Option<&str>) -> Result<DeploymentSet> {
         let script = self.worker_name(env);
         validate_worker_name(&script)?;
 
@@ -209,47 +212,46 @@ impl Manifest {
 
         let env = self.get_environment(env)?;
 
-        let mut add_routed_deployments =
-            |route_config: &RouteConfig| -> Result<(), failure::Error> {
-                if route_config.is_zoned() {
-                    let zoned = deploy::ZonedTarget::build(&script, route_config)?;
-                    // This checks all of the configured routes for the wildcard ending and warns
-                    // the user that their site may not work as expected without it.
-                    if self.site.is_some() {
-                        let no_star_routes = zoned
-                            .routes
-                            .iter()
-                            .filter(|r| !r.pattern.ends_with('*'))
-                            .map(|r| r.pattern.as_str())
-                            .collect::<Vec<_>>();
-                        if !no_star_routes.is_empty() {
-                            StdOut::warn(&format!(
+        let mut add_routed_deployments = |route_config: &RouteConfig| -> Result<()> {
+            if route_config.is_zoned() {
+                let zoned = deploy::ZonedTarget::build(&script, route_config)?;
+                // This checks all of the configured routes for the wildcard ending and warns
+                // the user that their site may not work as expected without it.
+                if self.site.is_some() {
+                    let no_star_routes = zoned
+                        .routes
+                        .iter()
+                        .filter(|r| !r.pattern.ends_with('*'))
+                        .map(|r| r.pattern.as_str())
+                        .collect::<Vec<_>>();
+                    if !no_star_routes.is_empty() {
+                        StdOut::warn(&format!(
                             "The following routes in your configuration file should have a trailing * to apply the Worker on every path, otherwise your site will not behave as expected.\n{}",
                             no_star_routes.join("\n"))
                         );
-                        }
                     }
-
-                    deployments.push(DeployTarget::Zoned(zoned));
                 }
 
-                if route_config.is_zoneless() {
-                    let zoneless = deploy::ZonelessTarget::build(&script, route_config)?;
-                    deployments.push(DeployTarget::Zoneless(zoneless));
-                }
+                deployments.push(DeployTarget::Zoned(zoned));
+            }
 
-                Ok(())
-            };
+            if route_config.is_zoneless() {
+                let zoneless = deploy::ZonelessTarget::build(&script, route_config)?;
+                deployments.push(DeployTarget::Zoneless(zoneless));
+            }
+
+            Ok(())
+        };
 
         if let Some(env) = env {
             if let Some(env_route_cfg) =
-                env.route_config(self.account_id.clone(), self.zone_id.clone())
+                env.route_config(self.account_id.if_present().cloned(), self.zone_id.clone())
             {
                 add_routed_deployments(&env_route_cfg)
             } else {
                 let config = self.route_config();
                 if config.is_zoned() {
-                    failure::bail!("you must specify route(s) per environment for zoned deploys.");
+                    anyhow::bail!("you must specify route(s) per environment for zoned deploys.");
                 } else {
                     add_routed_deployments(&config)
                 }
@@ -260,7 +262,10 @@ impl Manifest {
 
         let crons = match env {
             Some(e) => {
-                let account_id = e.account_id.as_ref().unwrap_or(&self.account_id);
+                let account_id = e
+                    .account_id
+                    .as_ref()
+                    .or_else(|| self.account_id.if_present());
                 e.triggers
                     .as_ref()
                     .or_else(|| self.triggers.as_ref())
@@ -269,53 +274,45 @@ impl Manifest {
             None => self
                 .triggers
                 .as_ref()
-                .map(|t| (t.crons.as_slice(), &self.account_id)),
+                .map(|t| (t.crons.as_slice(), self.account_id.if_present())),
         };
 
         if let Some((crons, account)) = crons {
             let scheduled =
-                deploy::ScheduleTarget::build(account.clone(), script.clone(), crons.to_vec())?;
+                deploy::ScheduleTarget::build(account.cloned(), script.clone(), crons.to_vec())?;
             deployments.push(DeployTarget::Schedule(scheduled));
         }
 
-        if deployments.is_empty() {
-            failure::bail!("No deployments specified!")
+        let durable_objects = match env {
+            Some(e) => e.durable_objects.as_ref(),
+            None => self.durable_objects.as_ref(),
+        };
+
+        if durable_objects.is_none() && deployments.is_empty() {
+            anyhow::bail!("Please specify your deployment routes or `wrangler_dev = true` inside of your configuration file. For more information, see: https://developers.cloudflare.com/workers/cli-wrangler/configuration#keys")
         }
 
         Ok(deployments)
     }
 
-    pub fn get_account_id(&self, environment_name: Option<&str>) -> Result<String, failure::Error> {
+    pub fn get_account_id(&self, environment_name: Option<&str>) -> Result<String> {
         let environment = self.get_environment(environment_name)?;
-        let mut result = self.account_id.to_string();
         if let Some(environment) = environment {
             if let Some(account_id) = &environment.account_id {
-                result = account_id.to_string();
+                return Ok(account_id.to_string());
             }
         }
-        if result.is_empty() {
-            let mut msg = "Your configuration file is missing an account_id field".to_string();
-            if let Some(environment_name) = environment_name {
-                msg.push_str(&format!(" in [env.{}]", environment_name));
-            }
-            failure::bail!("{}", &msg)
-        } else {
-            Ok(result)
-        }
+        self.account_id.load().map(String::from)
     }
 
-    pub fn get_target(
-        &self,
-        environment_name: Option<&str>,
-        preview: bool,
-    ) -> Result<Target, failure::Error> {
+    pub fn get_target(&self, environment_name: Option<&str>, preview: bool) -> Result<Target> {
         if self.site.is_some() {
             match self.target_type {
                 TargetType::Rust => {
-                    failure::bail!(format!(
+                    anyhow::bail!(
                         "{} Workers Sites does not support Rust type projects.",
                         emoji::WARN
-                    ))
+                    )
                 }
                 TargetType::JavaScript => {
                     let error_message = format!(
@@ -324,10 +321,10 @@ impl Manifest {
                     );
                     if let Some(build) = &self.build {
                         if build.command.is_none() {
-                            failure::bail!(error_message)
+                            anyhow::bail!(error_message)
                         }
                     } else {
-                        failure::bail!(error_message)
+                        anyhow::bail!(error_message)
                     }
                 }
                 _ => {}
@@ -351,11 +348,15 @@ impl Manifest {
             // to include the name of the environment
             name: self.name.clone(), // Inherited
             kv_namespaces: get_namespaces(self.kv_namespaces.clone(), preview)?, // Not inherited
+            durable_objects: self.durable_objects.clone(), // Not inherited
+            migrations: None,        // TODO(soon) Allow migrations in wrangler.toml
             site: self.site.clone(), // Inherited
             vars: self.vars.clone(), // Not inherited
             text_blobs: self.text_blobs.clone(), // Inherited
             usage_model: self.usage_model, // Top level
             wasm_modules: self.wasm_modules.clone(),
+            compatibility_date: self.compatibility_date.clone(),
+            compatibility_flags: self.compatibility_flags.clone(),
         };
 
         let environment = self.get_environment(environment_name)?;
@@ -363,7 +364,7 @@ impl Manifest {
         if let Some(environment) = environment {
             target.name = self.worker_name(environment_name);
             if let Some(account_id) = &environment.account_id {
-                target.account_id = account_id.clone();
+                target.account_id = Some(account_id.clone()).into();
             }
             if let Some(webpack_config) = &environment.webpack_config {
                 target.webpack_config = Some(webpack_config.clone());
@@ -375,6 +376,9 @@ impl Manifest {
             // don't inherit kv namespaces because it is an anti-pattern to use the same namespaces across multiple environments
             target.kv_namespaces = get_namespaces(environment.kv_namespaces.clone(), preview)?;
 
+            // don't inherit durable object configuration
+            target.durable_objects = environment.durable_objects.clone();
+
             // inherit site configuration
             if let Some(site) = &environment.site {
                 target.site = Some(site.clone());
@@ -387,27 +391,24 @@ impl Manifest {
         Ok(target)
     }
 
-    pub fn get_environment(
-        &self,
-        environment_name: Option<&str>,
-    ) -> Result<Option<&Environment>, failure::Error> {
+    pub fn get_environment(&self, environment_name: Option<&str>) -> Result<Option<&Environment>> {
         // check for user-specified environment name
         if let Some(environment_name) = environment_name {
             if let Some(environment_table) = &self.env {
                 if let Some(environment) = environment_table.get(environment_name) {
                     Ok(Some(environment))
                 } else {
-                    failure::bail!(format!(
+                    anyhow::bail!(
                         "{} Could not find environment with name \"{}\"",
                         emoji::WARN,
                         environment_name
-                    ))
+                    )
                 }
             } else {
-                failure::bail!(format!(
+                anyhow::bail!(
                     "{} There are no environments specified in your configuration file",
                     emoji::WARN
-                ))
+                )
             }
         } else {
             Ok(None)
@@ -418,9 +419,6 @@ impl Manifest {
         let account_id_env = env::var("CF_ACCOUNT_ID").is_ok();
         let zone_id_env = env::var("CF_ZONE_ID").is_ok();
         let mut top_level_fields: Vec<String> = Vec::new();
-        if !account_id_env {
-            top_level_fields.push("account_id".to_string());
-        }
         if let Some(kv_namespaces) = &self.kv_namespaces {
             for kv_namespace in kv_namespaces {
                 top_level_fields.push(format!(
@@ -478,6 +476,7 @@ impl Manifest {
             let toml_msg = styles::highlight("wrangler.toml");
             let zone_id_msg = styles::highlight("zone_id");
             let dash_url = styles::url("https://dash.cloudflare.com");
+            let account_id_msg = styles::highlight("account_id");
 
             StdOut::help(&format!(
                 "You can find your {} in the right sidebar of a zone's overview tab at {}",
@@ -489,6 +488,10 @@ impl Manifest {
             StdOut::help(
                 &format!("You will need to update the following fields in the created {} file before continuing:", toml_msg)
             );
+            StdOut::help(&format!(
+                "You can find your {} in the right sidebar of your account's Workers page, and {} in the right sidebar of a zone's overview tab at {} (if you have only a workers.dev domain, you can skip adding the {} )",
+                account_id_msg, zone_id_msg, dash_url, zone_id_msg
+            ));
 
             if has_top_level_fields {
                 needs_new_line = true;
@@ -513,6 +516,88 @@ impl Manifest {
     }
 }
 
+#[derive(Debug, Clone, Default, PartialEq)]
+pub struct LazyAccountId(OnceCell<String>);
+
+impl Serialize for LazyAccountId {
+    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
+    where
+        S: serde::Serializer,
+    {
+        self.0.get().serialize(serializer)
+    }
+}
+
+impl<'de> Deserialize<'de> for LazyAccountId {
+    fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
+    where
+        D: serde::Deserializer<'de>,
+    {
+        Ok(match Option::<String>::deserialize(deserializer)? {
+            None => None,
+            Some(x) if x.is_empty() => None,
+            Some(x) => Some(x),
+        }
+        .into())
+    }
+}
+
+impl From<Option<String>> for LazyAccountId {
+    fn from(opt: Option<String>) -> Self {
+        let cell = OnceCell::new();
+        if let Some(val) = opt {
+            cell.set(val).unwrap();
+        }
+        Self(cell)
+    }
+}
+
+impl LazyAccountId {
+    /// Return the `account_id` in `wrangler.toml`, if present.
+    ///
+    /// Use this with caution; prefer `maybe_load` instead where possible.
+    fn if_present(&self) -> Option<&String> {
+        self.0.get()
+    }
+
+    /// If `account_id` can be inferred automatically, do so;
+    /// otherwise, return `None`.
+    ///
+    /// Note that *unlike* `load`, this will never prompt the user or warn.
+    pub(crate) fn maybe_load(&self) -> Option<String> {
+        if let Some(id) = self.0.get() {
+            return Some(id.to_owned());
+        }
+
+        if let Some(mut accounts) = GlobalUser::new()
+            .ok()
+            .and_then(|user| fetch_accounts(&user).ok())
+        {
+            if accounts.len() == 1 {
+                return Some(accounts.pop().unwrap().id);
+            }
+        }
+
+        None
+    }
+
+    /// Load the account ID, possibly prompting the user.
+    pub(crate) fn load(&self) -> Result<&String> {
+        self.0.get_or_try_init(|| {
+            let user = GlobalUser::new()?;
+            match fetch_accounts(&user)?.as_slice() {
+                [] => unreachable!("auth token without account?"),
+                [single] => Ok(single.id.clone()),
+                _multiple => {
+                    StdOut::user_error("You have multiple accounts.");
+                    whoami::display_account_id_maybe();
+                    anyhow::bail!("field `account_id` is required")
+                }
+            }
+        })
+    }
+}
+
 impl FromStr for Manifest {
     type Err = toml::de::Error;
 
@@ -521,7 +606,7 @@ impl FromStr for Manifest {
     }
 }
 
-fn read_config(config_path: &Path) -> Result<Config, failure::Error> {
+fn read_config(config_path: &Path) -> Result<Config> {
     let mut config = Config::new();
 
     let config_str = config_path
@@ -535,7 +620,7 @@ fn read_config(config_path: &Path) -> Result<Config, failure::Error> {
     Ok(config)
 }
 
-fn check_for_duplicate_names(manifest: &Manifest) -> Result<(), failure::Error> {
+fn check_for_duplicate_names(manifest: &Manifest) -> Result<()> {
     let mut names: HashSet<String> = HashSet::new();
     let mut duplicate_names: HashSet<String> = HashSet::new();
     names.insert(manifest.name.to_string());
@@ -561,12 +646,12 @@ fn check_for_duplicate_names(manifest: &Manifest) -> Result<(), failure::Error>
         _ => None,
     };
     if let Some(message) = duplicate_message {
-        failure::bail!(format!(
+        anyhow::bail!(
             "{} Each name in your configuration file must be unique, {}: {}",
             emoji::WARN,
             message,
             duplicate_name_string
-        ))
+        )
     }
     Ok(())
 }
@@ -574,7 +659,7 @@ fn check_for_duplicate_names(manifest: &Manifest) -> Result<(), failure::Error>
 fn get_namespaces(
     kv_namespaces: Option<Vec<ConfigKvNamespace>>,
     preview: bool,
-) -> Result<Vec<KvNamespace>, failure::Error> {
+) -> Result<Vec<KvNamespace>> {
     if let Some(namespaces) = kv_namespaces {
         namespaces.into_iter().map(|ns| {
             if preview {
@@ -589,7 +674,7 @@ fn get_namespaces(
                         binding: ns.binding.to_string(),
                     })
                 } else {
-                    failure::bail!("In order to preview a worker with KV namespaces, you must designate a preview_id in your configuration file for each KV namespace you'd like to preview.")
+                    anyhow::bail!("In order to preview a worker with KV namespaces, you must designate a preview_id in your configuration file for each KV namespace you'd like to preview.")
                 }
             } else if let Some(id) = &ns.id {
                 Ok(KvNamespace {
@@ -597,10 +682,36 @@ fn get_namespaces(
                     binding: ns.binding,
                 })
             } else {
-                failure::bail!("You must specify the namespace ID in the id field for the namespace with binding \"{}\"", &ns.binding)
+                anyhow::bail!("You must specify the namespace ID in the id field for the namespace with binding \"{}\"", &ns.binding)
             }
         }).collect()
     } else {
         Ok(Vec::new())
     }
 }
+
+#[cfg(test)]
+mod tests {
+    use super::*;
+
+    #[test]
+    fn generate() -> Result<()> {
+        let toml_path = Path::new(".");
+
+        let toml = Manifest::generate(
+            "test".to_string(),
+            Some(TargetType::JavaScript),
+            toml_path,
+            None,
+        )?;
+        assert_eq!(toml.name, "test".to_string());
+        assert_eq!(toml.target_type.to_string(), "javascript".to_string());
+        fs::remove_file(toml_path.with_file_name("wrangler.toml"))?;
+
+        let toml = Manifest::generate("test".to_string(), None, toml_path, None)?;
+        assert_eq!(toml.target_type.to_string(), "webpack".to_string());
+        fs::remove_file(toml_path.with_file_name("wrangler.toml"))?;
+
+        Ok(())
+    }
+}
