@@ -15,6 +15,10 @@ struct Metadata {
     pub bindings: Vec<Binding>,
     pub migrations: Option<ApiMigration>,
     pub usage_model: Option<UsageModel>,
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub compatibility_date: Option<String>,
+    #[serde(skip_serializing_if = "Vec::is_empty")]
+    pub compatibility_flags: Vec<String>,
 }
 
 pub fn build_form(
@@ -53,6 +57,8 @@ fn add_metadata(mut form: Form, assets: &ModulesAssets) -> Result<Form> {
         bindings: assets.bindings(),
         migrations: assets.migration.clone(),
         usage_model: assets.usage_model,
+        compatibility_date: assets.compatibility_date.clone(),
+        compatibility_flags: assets.compatibility_flags.clone(),
     });
 
     let metadata = Part::text(metadata_json.to_string())
